VERSION 5.00
Begin VB.MDIForm frmMDIBackground 
   BackColor       =   &H8000000C&
   Caption         =   "MDIForm1"
   ClientHeight    =   4875
   ClientLeft      =   1560
   ClientTop       =   1845
   ClientWidth     =   8970
   Icon            =   "frmMDIBackground.frx":0000
   LinkTopic       =   "MDIForm1"
   Begin VB.Menu mnuFile 
      Caption         =   "File"
      Begin VB.Menu mnuOpen 
         Caption         =   "&Open"
      End
      Begin VB.Menu mnuSave 
         Caption         =   "&Save"
      End
      Begin VB.Menu mnuSaveAs 
         Caption         =   "Save &As..."
      End
      Begin VB.Menu Sep0 
         Caption         =   "-"
      End
      Begin VB.Menu mnuExit 
         Caption         =   "&Exit"
      End
   End
   Begin VB.Menu mnuStyle 
      Caption         =   "Style"
      Begin VB.Menu mnuBkStyle 
         Caption         =   "Stretch Background"
         Index           =   0
      End
      Begin VB.Menu mnuBkStyle 
         Caption         =   "Center"
         Index           =   1
      End
      Begin VB.Menu mnuBkStyle 
         Caption         =   "Center In Background"
         Index           =   2
      End
      Begin VB.Menu mnuBkStyle 
         Caption         =   "Tile"
         Index           =   3
      End
      Begin VB.Menu mnuBkStyle 
         Caption         =   "Tile In Background"
         Index           =   4
      End
   End
End
Attribute VB_Name = "frmMDIBackground"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
  Option Explicit
  ' demo project showing how to use the API functions to create a
  ' background image for a MDI form.
  ' by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at
  ' your own risk.

  ' the routines contained in this module are generic enough that they
  ' can be used for any form or control that has a picture property
  
  ' this demo uses a resource file and will not be able to find the
  ' resources if the compiled exe isn't somewhere on the current path
  ' when this code is run in the IDE
  
Private Sub MDIForm_Load()
  ' set the default item
  mnuBkStyle_Click 2
End Sub

Private Sub mnuBkStyle_Click(Index As Integer)
  ' remove the check mark from the previous item and check the new item
  ' store the current item for use the next time a menu item is clicked
  
  Static nPrevIndex&
  
  mnuBkStyle(nPrevIndex).Checked = False
  mnuBkStyle(Index).Checked = True
  
  nPrevIndex = Index
  
  ' call the routine that does the work
  CreateFormPicture
  
End Sub

Private Sub MDIForm_Resize()
  ' call the routine that does the work
  CreateFormPicture
End Sub

Private Sub CreateFormPicture()
  ' we load the desired bitmaps from the resource file as needed.  the
  ' following conatants hold the resource ID values.  if you have a form
  ' that will be resized constantly, you might want to load and cache the
  ' bitmaps to speed redrawing.
  Const FOREGROUND_BITMAP As Long = 200&
  Const BACKGROUND_BITMAP As Long = 201&
  

  ' in a MDI parent form, the client area is a seperate child window.  we
  ' need to use the GetWindow API function to get the hWnd to this window
  ' since VB does not expose it.
  Dim hCleintArea&, rc As RECT
  
  hCleintArea = FindWindowEx(Me.hWnd, 0&, "MDIClient", vbNullChar)
  

  ' we need to invalidate the client rect for the image to be redrawn properly on some systems.
  ' first, we get the rect to invalidate
  Call GetClientRect(hCleintArea, rc)
  
  ' then, we invalidate the client rect so it will be redrawn
  Call InvalidateRect(hCleintArea, rc, API_TRUE)


  Select Case True
    Case mnuBkStyle(0).Checked '<- background only
      ' stretch the background bitmap to fit the client area of the form.
      ' we need to pass the resource ID of the background bitmap
      Me.Picture = StretchToBackground(hCleintArea, BACKGROUND_BITMAP)
    
    
    Case mnuBkStyle(1).Checked '<- foreground only
      ' center the foreground bitmap on the default background....
      ' first, we need to clear out the existing image
      Me.Picture = Nothing
      ' then we need to pass the resource ID of the foreground
      ' bitmap and the transparent color used for drawing the bitmap onto the background
      Me.Picture = CenterForeground(hCleintArea, FOREGROUND_BITMAP, vbMagenta)
    
    Case mnuBkStyle(2).Checked '<- foreground and background
      ' center the foreground bitmap on the background bitmap.
      ' we need to pass the resource IDs of the foreground and background bitmaps
      ' and the color we want to be transparent when the foreground bitmap is drawn onto the background
      Me.Picture = CenterForegroundOnBackground(hCleintArea, FOREGROUND_BITMAP, BACKGROUND_BITMAP, vbMagenta)
      
    Case mnuBkStyle(3).Checked '<- tiled foreground
      ' tile the foreground bitmap.
      ' we need to pass both resource IDs and the transparent color
      Me.Picture = TileForeground(hCleintArea, FOREGROUND_BITMAP, vbMagenta)
    
    Case mnuBkStyle(4).Checked '<- tiled foreground on background
      ' tile the foreground bitmap onto the background.
      ' we need to pass both resource IDs and the transparent color
      Me.Picture = TileForegroundOnBackground(hCleintArea, FOREGROUND_BITMAP, BACKGROUND_BITMAP, vbMagenta)
      
  End Select
  
  ' we need to invalidate the client rect again for the redraw to occur
  Call InvalidateRect(hCleintArea, rc, API_TRUE)


End Sub

Public Function StretchToBackground(ByVal hWndTarget&, ByVal nBackgroundResID&) As StdPicture
  ' stretch the background onto the client area
  
  Dim hOwnerDC&, hBackgroundDC&, hStretchDC&, hBackgroundBmp&, hPrevBackgroundBmp&, hPrevStretchBmp&
  Dim hBmpObj&, ptBkgroundSize As POINTL, Bmp As BITMAP, rectStretch As RECT

  ' create the DCs used for the operation
  hOwnerDC = GetDC(hWndTarget)
  hBackgroundDC = CreateCompatibleDC(hOwnerDC)
  hStretchDC = CreateCompatibleDC(hOwnerDC)
  ReleaseDC hWndTarget, hOwnerDC
  SaveDC hBackgroundDC
  SaveDC hStretchDC


  Dim hModule&
  
  ' for this demo, we get the resource from the exe when running in the IDE.
  ' this is because the resource is not compiled into the module pointed to
  ' by App.hInstance when this code is run in the IDE
  If RunningInIde() Then
    hModule = LoadLibrary("PMDIBackground.exe")
  Else
    hModule = App.hInstance
  End If

  If hModule = 0 Then MsgBox "Unable to find resources!", vbExclamation

  ' load the background bitmap
  hBackgroundBmp = LoadBitmapBynum(hModule, nBackgroundResID)

  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hBackgroundBmp, Len(Bmp), Bmp) Then
    With ptBkgroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""StretchToBackground""."
  End If

  ' save the previous bitmap when we select the new one into the DC
  hPrevBackgroundBmp = SelectObject(hBackgroundDC, hBackgroundBmp)
  
  ' free the library if we are running in the IDE
  If RunningInIde() Then
    Call FreeLibrary(hModule)
  End If
      
  ' get the rect of the window
  Call GetClientRect(hWndTarget, rectStretch)
  
  
  With rectStretch
    ' if either scrollbar is showing, add the width of the scrollbar to the client area dimentions
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_VSCROLL Then .right = .right + GetSystemMetrics(SM_CXVSCROLL)
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_HSCROLL Then .bottom = .bottom + GetSystemMetrics(SM_CYHSCROLL)
    
    ' create a new bitmap and select it into the stretch DC
    hPrevStretchBmp = SelectObject(hStretchDC, CreateCompatibleBitmap(hBackgroundDC, .right, .bottom))
    
    ' stretch the background bitmap onto the stretched DC
    Call StretchBlt(hStretchDC, 0&, 0&, .right, .bottom, hBackgroundDC, 0&, 0&, ptBkgroundSize.x, ptBkgroundSize.y, SRCCOPY)
  
    ' create a picture object and return it to the caller
    Set StretchToBackground = PictureFromDC(hStretchDC, 0, 0, .right, .bottom)
  End With
  
  ' clean up after ourselves
  DeleteObject SelectObject(hBackgroundDC, hPrevBackgroundBmp)
  RestoreDC hBackgroundDC, (-1&)
  DeleteDC hBackgroundDC
  
  DeleteObject SelectObject(hStretchDC, hPrevStretchBmp)
  RestoreDC hStretchDC, (-1&)
  DeleteDC hStretchDC

End Function

Public Function CenterForeground(ByVal hWndTarget&, ByVal nForgroundResID&, ByVal nTransparentColor&) As StdPicture
  ' center the forground bitmap onto the existing background
  
  Dim hOwnerDC&, hForgroundDC&, hBackgroundDC&, hStretchDC&
  Dim hBmpObj&, ptForegroundSize As POINTL, ptBkgroundSize As POINTL, Bmp As BITMAP
  Dim nClientWidth&, nClientHeight&, rectStretch As RECT, rectForeground As RECT
  Dim hBackgroundBmp&, hPrevBackgroundBmp&, hForegroundBmp&, hPrevForegroundBmp&, hPrevStretchBmp&

  ' create the DCs we will use
  hOwnerDC = GetDC(hWndTarget)
  hForgroundDC = CreateCompatibleDC(hOwnerDC)
  hBackgroundDC = CreateCompatibleDC(hOwnerDC)
  hStretchDC = CreateCompatibleDC(hOwnerDC)
  SaveDC hForgroundDC
  SaveDC hBackgroundDC
  SaveDC hStretchDC


  Dim hModule&
  
  ' for this demo, we get the resource from the exe when running in the IDE.
  ' this is because the resource is not compiled into the module pointed to
  ' by App.hInstance when this code is run in the IDE
  If RunningInIde() Then
    hModule = LoadLibrary("PMDIBackground.exe")
  Else
    hModule = App.hInstance
  End If

  If hModule = 0 Then MsgBox "Unable to find resources!", vbExclamation

  ' load the foreground bitmap
  hForegroundBmp = LoadBitmapBynum(hModule, nForgroundResID)
  
  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hForegroundBmp, Len(Bmp), Bmp) Then
    With ptForegroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""CenterForeground""."
  End If

  ' save the previous bitmap when we select the new one into the DC
  hPrevForegroundBmp = SelectObject(hForgroundDC, hForegroundBmp)
    
  
  ' free the library if we are running in the IDE
  If RunningInIde() Then
    Call FreeLibrary(hModule)
  End If
      
  ' get the rect of the window
  Call GetClientRect(hWndTarget, rectStretch)
  

  With rectStretch
    ' if either scrollbar is showing, add the width of the scrollbar to the client area dimentions
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_VSCROLL Then .right = .right + GetSystemMetrics(SM_CXVSCROLL)
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_HSCROLL Then .bottom = .bottom + GetSystemMetrics(SM_CYHSCROLL)
    
    ' store the client dimentions for later use
    nClientWidth = .right
    nClientHeight = .bottom
    
    ' create a new bitmap and select it into the stretch DC
    hPrevBackgroundBmp = SelectObject(hBackgroundDC, CreateCompatibleBitmap(hOwnerDC, .right, .bottom))
    

    ' copy the existing background bitmap onto the stretched DC
    Call BitBlt(hBackgroundDC, 0, 0, .right, .bottom, hOwnerDC, .left, .top, SRCCOPY)

    ' release the owner DC
    ReleaseDC hWndTarget, hOwnerDC

    ' if the overlay graphic doesn't fit in the client area.....
    If (.right < (ptForegroundSize.x + 20)) Or (.bottom < (ptForegroundSize.y + 20)) Then
      Dim hShrinkDC&, hPrevBitmap&, nShrinkWidth&, nShrinkHeight&
      
      ' determine which dimention is the smallest and adjust the values accordingly....
      Select Case True
        Case ((ptForegroundSize.x + 20) - .right) > ((ptForegroundSize.y + 20) - .bottom)
          nShrinkWidth = .right - 20
          nShrinkHeight = nShrinkWidth * (ptForegroundSize.y / ptForegroundSize.x)
        
        Case Else
          nShrinkHeight = .bottom - 20
          nShrinkWidth = nShrinkHeight * (ptForegroundSize.x / ptForegroundSize.y)

      End Select
      
      ' create a DC to shrink the foreground bitmap into
      hShrinkDC = CreateCompatibleDC(hForgroundDC)
      SaveDC hShrinkDC

      ' create a new bitmap and select it into the shrink DC
      hPrevBitmap = SelectObject(hShrinkDC, CreateCompatibleBitmap(hForgroundDC, nShrinkWidth, nShrinkHeight))

      ' shrink the bitmap onto the shrink DC
      Call StretchBlt(hShrinkDC, 0&, 0&, nShrinkWidth, nShrinkHeight, hForgroundDC, 0&, 0&, ptForegroundSize.x, ptForegroundSize.y, SRCCOPY)

      ' set the dimentions of the graphic
      With rectForeground
        .right = nShrinkWidth
        .bottom = nShrinkHeight
      End With

      ' calculate the location for the foreground bitmap
      .left = (.right \ 2) - (nShrinkWidth \ 2)
      .right = .left + nShrinkWidth
      .top = (.bottom \ 2) - (nShrinkHeight \ 2)
      .bottom = .top + nShrinkHeight

      ' copy the graphic onto the background using the transparent color
      TransparentBlt hBackgroundDC, rectStretch, hShrinkDC, rectForeground, nTransparentColor

      ' clean up the shrink DC
      DeleteObject SelectObject(hShrinkDC, hPrevBitmap)
      RestoreDC hShrinkDC, (-1&)
      DeleteDC hShrinkDC

    Else
      ' set the dimentions of the graphic
      With rectForeground
        .right = ptForegroundSize.x
        .bottom = ptForegroundSize.y
      End With

      ' calculate the location for the foreground bitmap
      .left = (.right \ 2) - (ptForegroundSize.x \ 2)
      .right = .left + ptForegroundSize.x
      .top = (.bottom \ 2) - (ptForegroundSize.y \ 2)
      .bottom = .top + ptForegroundSize.y

      ' copy the graphic onto the background using the transparent color
      TransparentBlt hBackgroundDC, rectStretch, hForgroundDC, rectForeground, nTransparentColor
    End If

  End With
  
  ' create a picture object and return it to the caller
  Set CenterForeground = PictureFromDC(hBackgroundDC, 0, 0, nClientWidth, nClientHeight)
  

  ' delete the memory DCs used for holding the bitmaps
  DeleteObject SelectObject(hForgroundDC, hPrevForegroundBmp)
  RestoreDC hForgroundDC, (-1&)
  DeleteDC hForgroundDC
  
  DeleteObject SelectObject(hBackgroundDC, hPrevBackgroundBmp)
  RestoreDC hBackgroundDC, (-1&)
  DeleteDC hBackgroundDC
  
  DeleteObject SelectObject(hStretchDC, hPrevStretchBmp)
  RestoreDC hStretchDC, (-1&)
  DeleteDC hStretchDC

End Function

Public Function CenterForegroundOnBackground(ByVal hWndTarget&, ByVal nForgroundResID&, ByVal nBackgroundResID&, ByVal nTransparentColor&) As StdPicture

  Dim hOwnerDC&, hForgroundDC&, hBackgroundDC&, hStretchDC&
  Dim hBmpObj&, ptForegroundSize As POINTL, ptBkgroundSize As POINTL, Bmp As BITMAP
  Dim nClientWidth&, nClientHeight&, rectStretch As RECT, rectForeground As RECT
  Dim hBackgroundBmp&, hPrevBackgroundBmp&, hForegroundBmp&, hPrevForegroundBmp&, hPrevStretchBmp&


  ' create the DCs we will use
  hOwnerDC = GetDC(hWndTarget)
  hForgroundDC = CreateCompatibleDC(hOwnerDC)
  hBackgroundDC = CreateCompatibleDC(hOwnerDC)
  hStretchDC = CreateCompatibleDC(hOwnerDC)
  ReleaseDC hWndTarget, hOwnerDC
  SaveDC hForgroundDC
  SaveDC hBackgroundDC
  SaveDC hStretchDC


  Dim hModule&
  
  ' for this demo, we get the resource from the exe when running in the IDE.
  ' this is because the resource is not compiled into the module pointed to
  ' by App.hInstance when this code is run in the IDE
  If RunningInIde() Then
    hModule = LoadLibrary("PMDIBackground.exe")
  Else
    hModule = App.hInstance
  End If

  If hModule = 0 Then MsgBox "Unable to find resources!", vbExclamation

  ' load the foreground bitmap
  hForegroundBmp = LoadBitmapBynum(hModule, nForgroundResID)
  
  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hForegroundBmp, Len(Bmp), Bmp) Then
    With ptForegroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""CenterForegroundOnBackground""."
  End If

  ' save the previous bitmap when we select the new one into the DC
  hPrevForegroundBmp = SelectObject(hForgroundDC, hForegroundBmp)
  
  ' load the background bitmap
  hBackgroundBmp = LoadBitmapBynum(hModule, nBackgroundResID)

  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hBackgroundBmp, Len(Bmp), Bmp) Then
    With ptBkgroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""CenterForegroundOnBackground""."
  End If

  ' save the previous bitmap when we select the new one into the DC
  hPrevBackgroundBmp = SelectObject(hBackgroundDC, hBackgroundBmp)
  
  
  ' free the library if we are running in the IDE
  If RunningInIde() Then
    Call FreeLibrary(hModule)
  End If
      
  ' get the rect of the window
  Call GetClientRect(hWndTarget, rectStretch)
  

  With rectStretch
    ' if either scrollbar is showing, add the width of the scrollbar to the client area dimentions
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_VSCROLL Then .right = .right + GetSystemMetrics(SM_CXVSCROLL)
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_HSCROLL Then .bottom = .bottom + GetSystemMetrics(SM_CYHSCROLL)
    
    ' store the client dimentions for later use
    nClientWidth = .right
    nClientHeight = .bottom
    
    ' create a new bitmap and select it into the stretch DC
    hPrevStretchBmp = SelectObject(hStretchDC, CreateCompatibleBitmap(hBackgroundDC, .right, .bottom))
    

    ' stretch the background bitmap onto the stretched DC
    Call StretchBlt(hStretchDC, 0&, 0&, .right, .bottom, hBackgroundDC, 0&, 0&, ptBkgroundSize.x, ptBkgroundSize.y, SRCCOPY)


    ' if the overlay graphic doesn't fit in the client area.....
    If (.right < (ptForegroundSize.x + 20)) Or (.bottom < (ptForegroundSize.y + 20)) Then
      Dim hShrinkDC&, hPrevBitmap&, nShrinkWidth&, nShrinkHeight&
      
      
      ' determine which dimention is the smallest and adjust the values accordingly....
      Select Case True
        Case ((ptForegroundSize.x + 20) - .right) > ((ptForegroundSize.y + 20) - .bottom)
          nShrinkWidth = .right - 20
          nShrinkHeight = nShrinkWidth * (ptForegroundSize.y / ptForegroundSize.x)
        
        Case Else
          nShrinkHeight = .bottom - 20
          nShrinkWidth = nShrinkHeight * (ptForegroundSize.x / ptForegroundSize.y)

      End Select

      hShrinkDC = CreateCompatibleDC(hForgroundDC)
      SaveDC hShrinkDC

      ' create a new bitmap and select it into the shrink DC
      hPrevBitmap = SelectObject(hShrinkDC, CreateCompatibleBitmap(hForgroundDC, nShrinkWidth, nShrinkHeight))

      ' shrink the bitmap onto the shrink DC
      Call StretchBlt(hShrinkDC, 0&, 0&, nShrinkWidth, nShrinkHeight, hForgroundDC, 0&, 0&, ptForegroundSize.x, ptForegroundSize.y, SRCCOPY)

      ' set the dimentions of the graphic
      With rectForeground
        .right = nShrinkWidth
        .bottom = nShrinkHeight
      End With

      ' calculate the location for the foreground bitmap
      .left = (.right \ 2) - (nShrinkWidth \ 2)
      .right = .left + nShrinkWidth
      .top = (.bottom \ 2) - (nShrinkHeight \ 2)
      .bottom = .top + nShrinkHeight

      ' copy the graphic onto the background
      TransparentBlt hStretchDC, rectStretch, hShrinkDC, rectForeground, nTransparentColor

      ' clean up the shrink DC
      DeleteObject SelectObject(hShrinkDC, hPrevBitmap)
      RestoreDC hShrinkDC, (-1&)
      DeleteDC hShrinkDC

    Else
      ' set the dimentions of the graphic
      With rectForeground
        .right = ptForegroundSize.x
        .bottom = ptForegroundSize.y
      End With

      ' calculate the location for the foreground bitmap
      .left = (.right \ 2) - (ptForegroundSize.x \ 2)
      .right = .left + ptForegroundSize.x
      .top = (.bottom \ 2) - (ptForegroundSize.y \ 2)
      .bottom = .top + ptForegroundSize.y

      ' copy the graphic onto the background
      TransparentBlt hStretchDC, rectStretch, hForgroundDC, rectForeground, nTransparentColor
    End If

  End With
  

  ' create a picture object and return it to the caller
  Set CenterForegroundOnBackground = PictureFromDC(hStretchDC, 0, 0, nClientWidth, nClientHeight)
  

  ' delete the memory DCs used for holding the bitmaps
  DeleteObject SelectObject(hForgroundDC, hPrevForegroundBmp)
  RestoreDC hForgroundDC, (-1&)
  DeleteDC hForgroundDC
  
  DeleteObject SelectObject(hBackgroundDC, hPrevBackgroundBmp)
  RestoreDC hBackgroundDC, (-1&)
  DeleteDC hBackgroundDC
  
  DeleteObject SelectObject(hStretchDC, hPrevStretchBmp)
  RestoreDC hStretchDC, (-1&)
  DeleteDC hStretchDC

End Function

Public Function TileForeground(ByVal hWndTarget&, ByVal nForgroundResID&, ByVal nTransparentColor&) As StdPicture

  Dim hOwnerDC&, hForgroundDC&, hBackgroundDC&
  Dim hBmpObj&, ptForegroundSize As POINTL, ptBkgroundSize As POINTL, Bmp As BITMAP
  Dim nClientWidth&, nClientHeight&, rectStretch As RECT, rectForeground As RECT
  Dim hBackgroundBmp&, hPrevBackgroundBmp&, hForegroundBmp&, hPrevForegroundBmp&, hPrevStretchBmp&
  Dim rectTile As RECT

  ' create the DCs we will use
  hOwnerDC = GetDC(hWndTarget)
  hForgroundDC = CreateCompatibleDC(hOwnerDC)
  hBackgroundDC = CreateCompatibleDC(hOwnerDC)
  SaveDC hForgroundDC
  SaveDC hBackgroundDC


  Dim hModule&
  
  ' for this demo, we get the resource from the exe when running in the IDE.
  ' this is because the resource is not compiled into the module pointed to
  ' by App.hInstance when this code is run in the IDE
  If RunningInIde() Then
    hModule = LoadLibrary("PMDIBackground.exe")
  Else
    hModule = App.hInstance
  End If

  If hModule = 0 Then MsgBox "Unable to find resources!", vbExclamation

  ' load the foreground bitmap
  hForegroundBmp = LoadBitmapBynum(hModule, nForgroundResID)
  
  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hForegroundBmp, Len(Bmp), Bmp) Then
    With ptForegroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""TileForeground""."
  End If

  ' save the previous bitmap when we select the new one into the DC
  hPrevForegroundBmp = SelectObject(hForgroundDC, hForegroundBmp)
    
  
  ' free the library if we are running in the IDE
  If RunningInIde() Then
    Call FreeLibrary(hModule)
  End If
      
  ' get the rect of the window
  Call GetClientRect(hWndTarget, rectStretch)
  

  With rectStretch
    ' if either scrollbar is showing, add the width of the scrollbar to the client area dimentions
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_VSCROLL Then .right = .right + GetSystemMetrics(SM_CXVSCROLL)
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_HSCROLL Then .bottom = .bottom + GetSystemMetrics(SM_CYHSCROLL)
    
    ' store the client dimentions for later use
    nClientWidth = .right
    nClientHeight = .bottom
    
    ' clear the existing picture
    Set Me.Picture = Nothing
    
    ' create a new bitmap and select it into the stretch DC
    hPrevBackgroundBmp = SelectObject(hBackgroundDC, CreateCompatibleBitmap(hOwnerDC, .right, .bottom))
    

    ' copy the existing background bitmap onto the stretched DC
    Call BitBlt(hBackgroundDC, 0, 0, .right, .bottom, hOwnerDC, .left, .top, SRCCOPY)

    ' release the owner DC
    ReleaseDC hWndTarget, hOwnerDC

    ' if the overlay graphic doesn't fit in the client area.....
    If (.right < (ptForegroundSize.x + 20)) Or (.bottom < (ptForegroundSize.y + 20)) Then
      Dim hShrinkDC&, hPrevBitmap&, nShrinkWidth&, nShrinkHeight&
      
      ' determine which dimention is the smallest and adjust the values accordingly....
      Select Case True
        Case ((ptForegroundSize.x + 20) - .right) > ((ptForegroundSize.y + 20) - .bottom)
          nShrinkWidth = .right - 20
          nShrinkHeight = nShrinkWidth * (ptForegroundSize.y / ptForegroundSize.x)
        
        Case Else
          nShrinkHeight = .bottom - 20
          nShrinkWidth = nShrinkHeight * (ptForegroundSize.x / ptForegroundSize.y)

      End Select
      
      ' create a DC to shrink the foreground bitmap into
      hShrinkDC = CreateCompatibleDC(hForgroundDC)
      SaveDC hShrinkDC

      ' create a new bitmap and select it into the shrink DC
      hPrevBitmap = SelectObject(hShrinkDC, CreateCompatibleBitmap(hForgroundDC, nShrinkWidth, nShrinkHeight))

      ' shrink the bitmap onto the shrink DC
      Call StretchBlt(hShrinkDC, 0&, 0&, nShrinkWidth, nShrinkHeight, hForgroundDC, 0&, 0&, ptForegroundSize.x, ptForegroundSize.y, SRCCOPY)

      ' set the dimentions of the graphic
      With rectForeground
        .right = nShrinkWidth
        .bottom = nShrinkHeight
      End With

      ' calculate the location for the foreground bitmap
      .left = (.right \ 2) - (nShrinkWidth \ 2)
      .right = .left + nShrinkWidth
      .top = (.bottom \ 2) - (nShrinkHeight \ 2)
      .bottom = .top + nShrinkHeight

      ' copy the graphic onto the background using the transparent color
      TransparentBlt hBackgroundDC, rectStretch, hShrinkDC, rectForeground, nTransparentColor

      ' clean up the shrink DC
      DeleteObject SelectObject(hShrinkDC, hPrevBitmap)
      RestoreDC hShrinkDC, (-1&)
      DeleteDC hShrinkDC

    Else
      ' set the dimentions of the graphic
      With rectForeground
        .right = ptForegroundSize.x
        .bottom = ptForegroundSize.y
      End With

      ' copy the graphic onto the background
      TransparentBlt hBackgroundDC, rectForeground, hForgroundDC, rectForeground, nTransparentColor
      
      
      ' tile the images onto the new bitmap we create above
      With rectTile
        Do Until .top >= nClientHeight
            .bottom = .top + ptForegroundSize.y
            
          Do Until .left >= nClientWidth
          
            .right = .left + ptForegroundSize.x
            
            Call BitBlt(hBackgroundDC, .left, .top, .right, .bottom, hBackgroundDC, 0&, 0&, SRCCOPY)
              
            .left = .right
          Loop
          
          .left = 0
          .top = .bottom
        Loop
      End With
    End If

  End With
  
  ' create a picture object and return it to the caller
  Set TileForeground = PictureFromDC(hBackgroundDC, 0, 0, nClientWidth, nClientHeight)
  

  ' delete the memory DCs used for holding the bitmaps
  DeleteObject SelectObject(hForgroundDC, hPrevForegroundBmp)
  RestoreDC hForgroundDC, (-1&)
  DeleteDC hForgroundDC
  
  DeleteObject SelectObject(hBackgroundDC, hPrevBackgroundBmp)
  RestoreDC hBackgroundDC, (-1&)
  DeleteDC hBackgroundDC

End Function

Public Function TileForegroundOnBackground(ByVal hWndTarget&, ByVal nForgroundResID&, ByVal nBackgroundResID&, ByVal nTransparentColor&) As StdPicture

  Dim hOwnerDC&, hForgroundDC&, hBackgroundDC&, hStretchDC&
  Dim hBmpObj&, ptForegroundSize As POINTL, ptBkgroundSize As POINTL, Bmp As BITMAP
  Dim nClientWidth&, nClientHeight&, rectStretch As RECT, rectForeground As RECT
  Dim hBackgroundBmp&, hPrevBackgroundBmp&, hForegroundBmp&, hPrevForegroundBmp&, hPrevStretchBmp&
  Dim rectTile As RECT
  
  ' create the DCs we will use
  hOwnerDC = GetDC(hWndTarget)
  hForgroundDC = CreateCompatibleDC(hOwnerDC)
  hBackgroundDC = CreateCompatibleDC(hOwnerDC)
  hStretchDC = CreateCompatibleDC(hOwnerDC)
  ReleaseDC hWndTarget, hOwnerDC
  SaveDC hForgroundDC
  SaveDC hBackgroundDC
  SaveDC hStretchDC


  Dim hModule&
  
  ' for this demo, we get the resource from the exe when running in the IDE.
  ' this is because the resource is not compiled into the module pointed to
  ' by App.hInstance when this code is run in the IDE
  If RunningInIde() Then
    hModule = LoadLibrary("PMDIBackground.exe")
  Else
    hModule = App.hInstance
  End If

  If hModule = 0 Then MsgBox "Unable to find resources!", vbExclamation

  ' load the foreground bitmap
  hForegroundBmp = LoadBitmapBynum(hModule, nForgroundResID)
  
  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hForegroundBmp, Len(Bmp), Bmp) Then
    With ptForegroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""TileForegroundOnBackground""."
  End If
  
  ' save the previous bitmap when we select the new one into the DC
  hPrevForegroundBmp = SelectObject(hForgroundDC, hForegroundBmp)
  
  ' load the background bitmap
  hBackgroundBmp = LoadBitmapBynum(hModule, nBackgroundResID)

  ' get the bitmap info so that we can get the size of the bitmap
  If GetObjectAPI(hBackgroundBmp, Len(Bmp), Bmp) Then
    With ptBkgroundSize
      .x = Bmp.bmWidth
      .y = Bmp.bmHeight
    End With
  Else
    MsgBox "Unable to retrieve bitmap info in ""TileForegroundOnBackground""."
  End If

  ' save the previous bitmap when we select the new one into the DC
  hPrevBackgroundBmp = SelectObject(hBackgroundDC, hBackgroundBmp)
  
  
  ' free the library if we are running in the IDE
  If RunningInIde() Then
    Call FreeLibrary(hModule)
  End If
      
  ' get the rect of the window
  Call GetClientRect(hWndTarget, rectStretch)
  

  With rectStretch
    ' if either scrollbar is showing, add the width of the scrollbar to the client area dimentions
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_VSCROLL Then .right = .right + GetSystemMetrics(SM_CXVSCROLL)
    If GetWindowLong(hWndTarget, GWL_STYLE) And WS_HSCROLL Then .bottom = .bottom + GetSystemMetrics(SM_CYHSCROLL)
    
    ' store the client dimentions for later use
    nClientWidth = .right
    nClientHeight = .bottom
    
    ' create a new bitmap and select it into the stretch DC
    hPrevStretchBmp = SelectObject(hStretchDC, CreateCompatibleBitmap(hBackgroundDC, .right, .bottom))
    

    ' stretch the background bitmap onto the stretched DC
    Call StretchBlt(hStretchDC, 0&, 0&, .right, .bottom, hBackgroundDC, 0&, 0&, ptBkgroundSize.x, ptBkgroundSize.y, SRCCOPY)

    ' delete the background bitmap
    DeleteObject SelectObject(hBackgroundDC, hPrevBackgroundBmp)
    
    ' create a new bitmap to hold the tiled images
    hPrevBackgroundBmp = SelectObject(hBackgroundDC, CreateCompatibleBitmap(hStretchDC, .right, .bottom))


    ' set the dimentions of the graphic
    With rectForeground
      .right = ptForegroundSize.x
      .bottom = ptForegroundSize.y
    End With
    
    ' tile the images onto the new bitmap we create above
    With rectTile
      Do Until .top >= nClientHeight
          .bottom = .top + ptForegroundSize.y
          
        Do Until .left >= nClientWidth
        
          .right = .left + ptForegroundSize.x
          
          Call BitBlt(hBackgroundDC, .left, .top, .right, .bottom, hForgroundDC, 0&, 0&, SRCCOPY)
            
          .left = .right
        Loop
        
        .left = 0
        .top = .bottom
      Loop
    End With

  End With
  
  ' copy the tiled bitmap onto the background image
  TransparentBlt hStretchDC, rectStretch, hBackgroundDC, rectStretch, nTransparentColor


  ' create a picture object and return it to the caller
  Set TileForegroundOnBackground = PictureFromDC(hStretchDC, 0, 0, nClientWidth, nClientHeight)
  

  ' delete the memory DCs used for holding the bitmaps
  DeleteObject SelectObject(hForgroundDC, hPrevForegroundBmp)
  RestoreDC hForgroundDC, (-1&)
  DeleteDC hForgroundDC
  
  DeleteObject SelectObject(hBackgroundDC, hPrevBackgroundBmp)
  RestoreDC hBackgroundDC, (-1&)
  DeleteDC hBackgroundDC
  
  DeleteObject SelectObject(hStretchDC, hPrevStretchBmp)
  RestoreDC hStretchDC, (-1&)
  DeleteDC hStretchDC

End Function

Private Function PictureFromDC(ByVal hDCSrc&, ByVal nLeft&, ByVal nTop&, ByVal nWidth&, ByVal nHeight&) As StdPicture

  Dim hDCMemory&, hBmp&, hBmpPrev&, hPal&, hPalPrev&
  Dim fHasPalette&, nPaletteEntries&, LogPal As LOGPALETTE256

  ' create the DC and bitmap we will use
  hDCMemory = CreateCompatibleDC(hDCSrc)
  hBmp = CreateCompatibleBitmap(hDCSrc, nWidth, nHeight)
  hBmpPrev = SelectObject(hDCMemory, hBmp)

  ' determine whether or not the video supports 256 color palettes
  nPaletteEntries = GetDeviceCaps(hDCSrc, SIZEPALETTE)
  fHasPalette = GetDeviceCaps(hDCSrc, RASTERCAPS) And RC_PALETTE

  ' if this is 256 color video, we need to create and add a palette to our DC
  If fHasPalette And (nPaletteEntries = 256) Then
    LogPal.palVersion = &H300
    LogPal.palNumEntries = 256
    GetSystemPaletteEntries hDCSrc, 0, 256, LogPal.palPalEntry(0)
    hPal = CreatePalette(LogPal)
    hPalPrev = SelectPalette(hDCMemory, hPal, 0)
    RealizePalette hDCMemory
  End If

  ' copy the passed image into the local DC
  BitBlt hDCMemory, 0, 0, nWidth, nHeight, hDCSrc, nLeft, nTop, vbSrcCopy

  ' get the bitmap from the DC
  hBmp = SelectObject(hDCMemory, hBmpPrev)

  ' if we created a palette, get it from the DC
  If fHasPalette And (nPaletteEntries = 256) Then
    hPal = SelectPalette(hDCMemory, hPalPrev, 0)
  End If

  ' clean up
  DeleteDC hDCMemory

  ' create a picture from the bitmap and return it to the caller
  Set PictureFromDC = PictureFromBitmap(hBmp, hPal)
    
End Function

Private Function PictureFromBitmap(ByVal hBmp&, ByVal hPal&) As StdPicture

  Dim IPictureIID As GUID, IPic As IPicture, tagPic As PICTDESC_BMP
  Dim lpGUID&
  
  ' fill in the IPicture GUID {7BF80980-BF32-101A-8BBB-00AA00300CAB}
  With IPictureIID
    .Data1 = &H7BF80980
    .Data2 = &HBF32
    .Data3 = &H101A
    .Data4(0) = &H8B
    .Data4(1) = &HBB
    .Data4(2) = &H0
    .Data4(3) = &HAA
    .Data4(4) = &H0
    .Data4(5) = &H30
    .Data4(6) = &HC
    .Data4(7) = &HAB
  End With
  
  
  ' set the properties on the picture object
  With tagPic
    .Size = Len(tagPic)
    .Type = vbPicTypeBitmap
    .hBmp = hBmp
    .hPal = hPal
  End With

  ' create a picture that will delete it's bitmap when it is finished with it
  OleCreatePictureIndirect tagPic, IPictureIID, API_TRUE, IPic

  ' return the picture to the caller
  Set PictureFromBitmap = IPic
    
End Function

Private Sub TransparentBlt(hDestDC&, lpDestRect As RECT, hSrcDC&, lpSrcRect As RECT, ByVal TransColor&)

  Dim hInvDC&, hMaskDC&, hResultDC&, hInvBmp&, hMaskBmp&, hResultBmp&
  Dim hInvPrevBmp&, hMaskPrevBmp&, hDestPrevBmp&, nSrcWidth&, nSrcHeight&, nOriginalColor&

  With lpSrcRect
    nSrcWidth = .right - .left
    nSrcHeight = .bottom - .top
  End With
  
  ' create the mask and invert stage DCs and bitmaps
  hInvDC = CreateCompatibleDC(hDestDC)
  hMaskDC = CreateCompatibleDC(hDestDC)
  ' monochrome bitmaps for the masks
  hInvBmp = CreateBitmap(nSrcWidth, nSrcHeight, 1, 1, ByVal 0&)
  hMaskBmp = CreateBitmap(nSrcWidth, nSrcHeight, 1, 1, ByVal 0&)
  hInvPrevBmp = SelectObject(hInvDC, hInvBmp)
  hMaskPrevBmp = SelectObject(hMaskDC, hMaskBmp)

  ' create the DC and bitmap to hold the result
  hResultDC = CreateCompatibleDC(hDestDC)
  ' color bitmap for final result
  hResultBmp = CreateCompatibleBitmap(hDestDC, nSrcWidth, nSrcHeight)
  hDestPrevBmp = SelectObject(hResultDC, hResultBmp)

  ' create mask: set background color of source to transparent color.
  nOriginalColor = SetBkColor(hSrcDC, TransColor)
  With lpSrcRect
    Call BitBlt(hMaskDC, 0, 0, nSrcWidth, nSrcHeight, hSrcDC, .left, .top, SRCCOPY)
  End With
  TransColor = SetBkColor(hSrcDC, nOriginalColor)

  ' create inverse of mask to AND w/ source & combine w/ background.
  Call BitBlt(hInvDC, 0, 0, nSrcWidth, nSrcHeight, hMaskDC, 0, 0, NOTSRCCOPY)

  ' copy background bitmap to result & create final transparent bitmap
  With lpDestRect
    Call BitBlt(hResultDC, 0, 0, nSrcWidth, nSrcHeight, hDestDC, .left, .top, SRCCOPY)
  
    ' AND mask bitmap w/ result DC to punch hole in the background by
    ' painting black area for non-transparent portion of source bitmap.
    Call BitBlt(hResultDC, 0, 0, nSrcWidth, nSrcHeight, hMaskDC, 0, 0, SRCAND)
  
    ' AND inverse mask w/ source bitmap to turn off bits associated
    ' with transparent area of source bitmap by making it black.
    Call BitBlt(hSrcDC, 0, 0, nSrcWidth, nSrcHeight, hInvDC, 0, 0, SRCAND)
  
    ' XOR result w/ source bitmap to make background show through.
    Call BitBlt(hResultDC, 0, 0, nSrcWidth, nSrcHeight, hSrcDC, 0, 0, SRCPAINT)
  
    ' copy result to the dest DC
    Call BitBlt(hDestDC, .left, .top, nSrcWidth, nSrcHeight, hResultDC, 0, 0, SRCCOPY)
  End With

  ' clean up after ourselves
  Call DeleteObject(SelectObject(hMaskDC, hMaskPrevBmp))
  Call DeleteObject(SelectObject(hInvDC, hInvPrevBmp))
  Call DeleteObject(SelectObject(hResultDC, hDestPrevBmp))

  DeleteDC hMaskDC
  DeleteDC hInvDC
  DeleteDC hResultDC

End Sub

Private Function RunningInIde() As Boolean
  
  Dim sClassName$, nStrLen&
  ' check to see where we're running in the IDE
  
  sClassName = String$(260, vbNullChar)
  nStrLen = GetClassName(hWnd, sClassName, Len(sClassName))
  If nStrLen Then sClassName = left$(sClassName, nStrLen)
  
  If sClassName = "ThunderMDIForm" Then RunningInIde = True
  
End Function
